@@ -88,10 +88,15 @@ pub enum BackgroundProcessingEvent {
 
 #[derive(Debug)]
 pub enum AppEvent {
-    Rerender,
     PerformSearch,
 }
 
+#[derive(Debug)]
+pub enum InternalEvent {
+    App(AppEvent),
+    Background(BackgroundProcessingEvent),
+}
+
 #[derive(Debug)]
 pub struct ExitAndReplaceState {
     pub stdin: Arc<String>,
@@ -102,9 +107,9 @@ pub struct ExitAndReplaceState {
 #[derive(Debug)]
 pub enum Event {
     LaunchEditor((PathBuf, usize)),
-    App(AppEvent),
-    PerformReplacement,
     ExitAndReplace(ExitAndReplaceState),
+    Rerender,
+    Internal(InternalEvent),
 }
 
 #[derive(Debug, PartialEq, Eq)]
@@ -470,6 +475,7 @@ pub struct App {
     pub searcher: Option<Searcher>,
     pub input_source: InputSource,
     pub event_sender: UnboundedSender<Event>,
+    event_receiver: UnboundedReceiver<Event>,
     errors: Vec<AppError>,
     include_hidden: bool,
     immediate_replace: bool,
@@ -488,14 +494,63 @@ enum SearchStrategy {
     },
 }
 
+fn generate_escape_deprecation_message(quit_keymap: Option<KeyEvent>) -> String {
+    let quit_keymap_str = quit_keymap.map_or("".to_string(), |keymap| {
+        let optional_help = if let KeyEvent {
+            code: KeyCode::Char('c'),
+            modifiers: KeyModifiers::CONTROL,
+        } = keymap
+        {
+            // Add some additional help text when using the default
+            " (i.e. `ctrl + c`)"
+        } else {
+            ""
+        };
+        format!(": use `{keymap}`{optional_help} instead")
+    });
+
+    format!(
+        "Pressing escape to quit is no longer enabled by default{quit_keymap_str}.\n\nYou can remap this in your scooter config.",
+    )
+}
+
+// Macro to get the background processing receiver from current_screen, needed because
+// methods can't express split borrows but macros can
+macro_rules! get_bg_receiver {
+    ($self:expr) => {
+        match &mut $self.current_screen {
+            Screen::SearchFields(SearchFieldsState { search_state, .. }) => {
+                search_state.as_mut().map(|s| &mut s.processing_receiver)
+            }
+            Screen::PerformingReplacement(PerformingReplacementState {
+                processing_receiver,
+                ..
+            }) => Some(processing_receiver),
+            Screen::Results(_) => None,
+        }
+    };
+}
+
+macro_rules! recv_optional {
+    ($opt_receiver:expr) => {
+        async {
+            match $opt_receiver {
+                Some(r) => r.recv().await,
+                None => None,
+            }
+        }
+    };
+}
+
 impl<'a> App {
-    fn new(
+    pub fn new(
         input_source: InputSource,
         search_field_values: &SearchFieldValues<'a>,
-        event_sender: UnboundedSender<Event>,
         app_run_config: &AppRunConfig,
         config: Config,
     ) -> anyhow::Result<Self> {
+        let (event_sender, event_receiver) = mpsc::unbounded_channel();
+
         let search_fields = SearchFields::with_values(
             search_field_values,
             config.search.disable_prepopulated_fields,
@@ -519,6 +574,7 @@ impl<'a> App {
             errors: vec![],
             popup: None,
             event_sender,
+            event_receiver,
             immediate_replace: app_run_config.immediate_replace,
             print_results: app_run_config.print_results,
             print_on_exit: app_run_config.print_on_exit,
@@ -532,21 +588,23 @@ impl<'a> App {
         Ok(app)
     }
 
-    pub fn new_with_receiver(
-        input_source: InputSource,
-        search_field_values: &SearchFieldValues<'a>,
-        app_run_config: &AppRunConfig,
-        config: Config,
-    ) -> anyhow::Result<(Self, UnboundedReceiver<Event>)> {
-        let (event_sender, app_event_receiver) = mpsc::unbounded_channel();
-        let app = Self::new(
-            input_source,
-            search_field_values,
-            event_sender,
-            app_run_config,
-            config,
-        )?;
-        Ok((app, app_event_receiver))
+    pub fn handle_internal_event(&mut self, event: InternalEvent) -> EventHandlingResult {
+        match event {
+            InternalEvent::App(app_event) => self.handle_app_event(app_event),
+            InternalEvent::Background(bg_event) => {
+                self.handle_background_processing_event(bg_event)
+            }
+        }
+    }
+
+    #[allow(clippy::needless_pass_by_value)]
+    fn handle_app_event(&mut self, app_event: AppEvent) -> EventHandlingResult {
+        match app_event {
+            AppEvent::PerformSearch => {
+                self.perform_search_unwrap();
+                EventHandlingResult::Rerender
+            }
+        }
     }
 
     fn cancel_search(&mut self) {
@@ -577,7 +635,6 @@ impl<'a> App {
         *self = Self::new(
             self.input_source.clone(), // TODO: avoid cloning
             &SearchFieldValues::default(),
-            self.event_sender.clone(), // TODO: avoid cloning
             &AppRunConfig {
                 include_hidden: self.include_hidden,
                 advanced_regex: self.advanced_regex,
@@ -591,62 +648,39 @@ impl<'a> App {
         .expect("App initialisation errors should have been detected on initial construction");
     }
 
-    pub async fn background_processing_recv(&mut self) -> Option<BackgroundProcessingEvent> {
-        match self.background_processing_reciever() {
-            Some(r) => r.recv().await,
-            None => None,
+    pub async fn event_recv(&mut self) -> Event {
+        tokio::select! {
+            Some(event) = self.event_receiver.recv() => event,
+            Some(bg_event) = recv_optional!(get_bg_receiver!(self)) => {
+                Event::Internal(InternalEvent::Background(bg_event))
+            }
         }
     }
 
     pub fn background_processing_reciever(
         &mut self,
     ) -> Option<&mut UnboundedReceiver<BackgroundProcessingEvent>> {
-        match &mut self.current_screen {
-            Screen::SearchFields(SearchFieldsState { search_state, .. }) => {
-                if let Some(search_state) = search_state {
-                    Some(&mut search_state.processing_receiver)
-                } else {
-                    None
-                }
-            }
-            Screen::PerformingReplacement(PerformingReplacementState {
-                processing_receiver,
-                ..
-            }) => Some(processing_receiver),
-            Screen::Results(_) => None,
-        }
+        get_bg_receiver!(self)
     }
 
-    pub fn handle_app_event(&mut self, event: &AppEvent) -> EventHandlingResult {
-        match event {
-            AppEvent::Rerender => EventHandlingResult::Rerender,
-            AppEvent::PerformSearch => {
-                if self.search_fields.search().text().is_empty() {
-                    if let Screen::SearchFields(ref mut search_fields_state) = self.current_screen {
-                        search_fields_state.search_state = None;
-                    }
-                    EventHandlingResult::Rerender
-                } else {
-                    self.perform_search_unwrap()
-                }
-            }
-        }
-    }
-
-    pub fn perform_search_if_valid(&mut self) -> EventHandlingResult {
-        if let Some(search_config) = self.validate_fields().unwrap() {
-            self.searcher = Some(search_config);
-        } else {
-            return EventHandlingResult::Rerender;
-        }
-        self.perform_search_unwrap()
+    // Called when searching explicitly: shows error popup if validation fails
+    pub fn perform_search_if_valid(&mut self) {
+        let Some(search_config) = self.validate_fields().unwrap() else {
+            return;
+        };
+        self.searcher = Some(search_config);
+        self.perform_search_unwrap();
     }
 
     /// NOTE: validation should have been performed (with `validate_fields`) before calling
-    fn perform_search_unwrap(&mut self) -> EventHandlingResult {
+    // TODO: how can we enforce validation by type system - e.g. pass in searcher?
+    fn perform_search_unwrap(&mut self) {
         let Screen::SearchFields(ref mut search_fields_state) = self.current_screen else {
-            return EventHandlingResult::None;
+            return;
         };
+        if self.search_fields.search().text().is_empty() {
+            search_fields_state.search_state = None;
+        }
 
         let (background_processing_sender, background_processing_receiver) =
             mpsc::unbounded_channel();
@@ -683,8 +717,6 @@ impl<'a> App {
         );
 
         search_fields_state.search_state = Some(search_state);
-
-        EventHandlingResult::Rerender
     }
 
     #[allow(clippy::needless_pass_by_value)]
@@ -758,10 +790,6 @@ impl<'a> App {
         EventHandlingResult::Rerender
     }
 
-    pub fn trigger_replacement(&mut self) {
-        let _ = self.event_sender.send(Event::PerformReplacement);
-    }
-
     pub fn perform_replacement(&mut self) {
         if !self.ready_to_replace() {
             return;
@@ -873,7 +901,7 @@ impl<'a> App {
                     state.set_search_completed_now();
                     if self.immediate_replace && *focussed_section == FocussedSection::SearchResults
                     {
-                        self.trigger_replacement();
+                        self.perform_replacement();
                     }
                 }
                 EventHandlingResult::Rerender
@@ -966,7 +994,7 @@ impl<'a> App {
                     // Check if search has been performed
                     if search_fields_state.search_state.is_some() {
                         if self.immediate_replace && self.search_has_completed() {
-                            self.trigger_replacement();
+                            self.perform_replacement();
                         }
                     } else {
                         if let Some(timer) = search_fields_state.search_debounce_timer.take() {
@@ -1061,7 +1089,8 @@ impl<'a> App {
             let event_sender = self.event_sender.clone();
             search_fields_state.search_debounce_timer = Some(tokio::spawn(async move {
                 tokio::time::sleep(Duration::from_millis(300)).await;
-                let _ = event_sender.send(Event::App(AppEvent::PerformSearch));
+                let _ =
+                    event_sender.send(Event::Internal(InternalEvent::App(AppEvent::PerformSearch)));
             }));
         }
         EventHandlingResult::Rerender
@@ -1089,16 +1118,13 @@ impl<'a> App {
 
         match event {
             CommandSearchFocusResults::TriggerReplacement => {
-                self.trigger_replacement();
+                self.perform_replacement();
                 EventHandlingResult::Rerender
             }
             CommandSearchFocusResults::BackToFields => {
                 self.cancel_search();
                 let search_fields_state = self.current_screen.unwrap_search_fields_state_mut();
                 search_fields_state.focussed_section = FocussedSection::SearchFields;
-                self.event_sender
-                    .send(Event::App(AppEvent::Rerender))
-                    .unwrap();
                 EventHandlingResult::Rerender
             }
             CommandSearchFocusResults::OpenInEditor => {
@@ -1265,9 +1291,10 @@ impl<'a> App {
             event
         } else {
             if key_event.code == KeyCode::Esc {
-                self.set_popup(Popup::Text{
+                let quit_keymap = self.config.keys.general.quit.first().copied();
+                self.set_popup(Popup::Text {
                     title: "Key mapping deprecated".to_string(),
-                    body: "Pressing escape to quit is no longer enabled by default: use `ctrl + c` instead.\n\nYou can remap this in your scooter config.".to_string(),
+                    body: generate_escape_deprecation_message(quit_keymap),
                 });
                 return Right(EventHandlingResult::Rerender);
             }
@@ -1393,7 +1420,7 @@ impl<'a> App {
                         break;
                     },
                     _ = rerender_interval.tick() => {
-                        let _ = event_sender.send(Event::App(AppEvent::Rerender));
+                        let _ = event_sender.send(Event::Rerender);
                     }
                 }
             }
@@ -2264,7 +2291,7 @@ mod tests {
 
     #[test]
     fn test_key_handling_quit_takes_precedent() {
-        let (mut app, _app_event_receiver) = App::new_with_receiver(
+        let mut app = App::new(
             InputSource::Directory(std::env::current_dir().unwrap()),
             &SearchFieldValues::default(),
             &AppRunConfig::default(),
@@ -2281,7 +2308,7 @@ mod tests {
 
     #[test]
     fn test_key_handling_unmapped_key_closes_popup() {
-        let (mut app, _app_event_receiver) = App::new_with_receiver(
+        let mut app = App::new(
             InputSource::Directory(std::env::current_dir().unwrap()),
             &SearchFieldValues::default(),
             &AppRunConfig::default(),
@@ -2296,4 +2323,50 @@ mod tests {
         assert!(matches!(res, EventHandlingResult::Rerender));
         assert!(app.popup().is_none());
     }
+
+    #[test]
+    fn test_escape_deprecation_message_with_default() {
+        let keymap = KeyEvent::new(KeyCode::Char('c'), KeyModifiers::CONTROL);
+        let message = generate_escape_deprecation_message(Some(keymap));
+        assert_eq!(
+            message,
+            "Pressing escape to quit is no longer enabled by default: use `C-c` \
+             (i.e. `ctrl + c`) instead.\n\nYou can remap this in your scooter config."
+        );
+    }
+
+    #[test]
+    fn test_escape_deprecation_message_with_no_mapping() {
+        let message = generate_escape_deprecation_message(None);
+        assert_eq!(
+            message,
+            "Pressing escape to quit is no longer enabled by default.\n\n\
+             You can remap this in your scooter config."
+        );
+    }
+
+    #[test]
+    fn test_escape_deprecation_message_with_f_key() {
+        let keymap = KeyEvent::new(KeyCode::F(1), KeyModifiers::NONE);
+        let message = generate_escape_deprecation_message(Some(keymap));
+        assert_eq!(
+            message,
+            "Pressing escape to quit is no longer enabled by default: use `F1` instead.\n\n\
+             You can remap this in your scooter config."
+        );
+    }
+
+    #[test]
+    fn test_escape_deprecation_message_with_ctrl_alt_q_keymap() {
+        let keymap = KeyEvent::new(
+            KeyCode::Char('q'),
+            KeyModifiers::CONTROL | KeyModifiers::ALT,
+        );
+        let message = generate_escape_deprecation_message(Some(keymap));
+        assert_eq!(
+            message,
+            "Pressing escape to quit is no longer enabled by default: use `C-A-q` instead.\n\n\
+             You can remap this in your scooter config."
+        );
+    }
 }
