@@ -1,7 +1,10 @@
 pub mod app;
+pub mod commands;
+pub mod config;
 pub mod diff;
 pub mod errors;
 pub mod fields;
+pub mod keyboard;
 pub mod replace;
 pub mod search;
 pub mod utils;
