@@ -30,7 +30,7 @@ use std::{
     str::FromStr,
     sync::Arc,
 };
-use tokio::sync::mpsc::{UnboundedReceiver, UnboundedSender};
+use tokio::sync::mpsc::UnboundedSender;
 
 use crate::{logging::DEFAULT_LOG_LEVEL, tui::Tui};
 
@@ -42,6 +42,7 @@ pub struct AppConfig<'a> {
     pub search_field_values: SearchFieldValues<'a>,
     pub app_run_config: AppRunConfig,
     pub stdin_content: Option<String>,
+    pub editor_command_override: Option<String>,
 }
 
 impl Default for AppConfig<'_> {
@@ -52,6 +53,7 @@ impl Default for AppConfig<'_> {
             search_field_values: SearchFieldValues::default(),
             app_run_config: AppRunConfig::default(),
             stdin_content: None,
+            editor_command_override: None,
         }
     }
 }
@@ -66,7 +68,6 @@ pub type CrosstermEventStream = event::EventStream;
 
 pub struct AppRunner<B: Backend, E: EventStream, S: SnapshotProvider<B>> {
     app: App,
-    event_receiver: UnboundedReceiver<Event>,
     tui: Tui<B>,
     event_stream: E,
     snapshot_provider: S,
@@ -122,7 +123,13 @@ impl AppRunner<CrosstermBackend<io::Stdout>, CrosstermEventStream, NoOpSnapshotP
         let backend = CrosstermBackend::new(io::stdout());
         let event_stream = CrosstermEventStream::new();
         let snapshot_provider = NoOpSnapshotProvider;
-        let user_config = config::load_config().context("Failed to read config file")?;
+        let mut user_config = config::load_config().context("Failed to read config file")?;
+
+        // Apply CLI override for editor command if provided
+        if let Some(ref editor_command) = app_config.editor_command_override {
+            user_config.editor_open.command = Some(editor_command.clone());
+        }
+
         Self::new(
             app_config,
             user_config,
@@ -181,7 +188,7 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
             InputSource::Directory(app_config.directory.clone())
         };
 
-        let (app, event_receiver) = App::new_with_receiver(
+        let app = App::new(
             input_source,
             &app_config.search_field_values,
             &app_config.app_run_config,
@@ -193,7 +200,6 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
 
         Ok(Self {
             app,
-            event_receiver,
             tui,
             event_stream,
             snapshot_provider,
@@ -227,7 +233,7 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
                         _ => EventHandlingResult::None,
                     }
                 }
-                Some(event) = self.event_receiver.recv() => {
+                event = self.app.event_recv() => {
                     match event {
                         Event::LaunchEditor((file_path, line)) => {
                             let mut res = EventHandlingResult::Rerender;
@@ -251,21 +257,17 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
                             self.tui.init()?;
                             res
                         }
-                        Event::App(app_event) => {
-                            self.app.handle_app_event(&app_event)
-                        }
-                        Event::PerformReplacement => {
-                            self.app.perform_replacement();
-                            EventHandlingResult::Rerender
-                        }
                         Event::ExitAndReplace(state) => {
                             return Ok(Some(ExitState::StdinState(state)));
                         }
+                        Event::Rerender => {
+                            EventHandlingResult::Rerender
+                        }
+                        Event::Internal(internal_event) => {
+                            self.app.handle_internal_event(internal_event)
+                        }
                     }
                 }
-                Some(event) = self.app.background_processing_recv() => {
-                    self.app.handle_background_processing_event(event)
-                }
                 else => {
                     return Ok(None);
                 }
